@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2020 J.F.Dockes
+/* Copyright (C) 2015-2022 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as published by
  *   the Free Software Foundation; either version 2.1 of the License, or
@@ -16,7 +16,6 @@
  */
 /////////////////////////////////////////////////////////////////////
 // Main program
-#define UPMPDCLI_NEED_PACKAGE_VERSION
 #include "config.h"
 
 #include <errno.h>             
@@ -46,30 +45,18 @@
 #include "upmpdutils.hxx"
 #include "pathut.h"
 #include "readfile.h"
+#include "smallut.h"
 
 using namespace std;
 using namespace UPnPP;
 
-// Can't remember why it's static in config.h and don't want to try change
-const string g_upmpdcli_package_version{UPMPDCLI_PACKAGE_VERSION};
-
 static char *thisprog;
 
 static int op_flags;
-#define OPT_MOINS 0x1   
-#define OPT_D     0x2   
-#define OPT_O     0x4   
-#define OPT_P     0x8   
-#define OPT_c     0x10  
-#define OPT_d     0x20  
-#define OPT_f     0x40  
-#define OPT_h     0x80  
-#define OPT_i     0x100 
-#define OPT_l     0x200 
-#define OPT_m     0x400 
-#define OPT_p     0x800 
-#define OPT_q     0x1000
-#define OPT_v     0x2000
+#define OPT_MOINS 0x1
+#define OPT_D     0x2
+#define OPT_c     0x4
+#define OPT_m     0x8
 
 static const char usage[] = 
     "-c configfile \t configuration file to use\n"
@@ -84,8 +71,8 @@ static const char usage[] =
     "-P upport    \t specify port number to be used for UPnP\n"
     "-O 0|1\t decide if we run and export the OpenHome services\n"
     "-v      \tprint version info\n"
-    "-m <0|1|2|3|4> media server mode "
-    "(default, multidev|only renderer|only media|embedded|multidev)\n"
+    "-m <0|1|2|3|4> media server mode. 0 or no option: default, currently equivalent to 4\n"
+    "    1: renderer only, 2: media server only,  3: combined embedded, 4: combined dual roots.\n"
     "\n"
     ;
 
@@ -107,23 +94,16 @@ static const char usage[] =
 
 enum MSMode {Default, RdrOnly, MSOnly, CombinedEmbedded, CombinedMultiDev};
 
-static void
-versionInfo(FILE *fp)
-{
-    fprintf(fp, "Upmpdcli %s %s\n",
-            UPMPDCLI_PACKAGE_VERSION, LibUPnP::versionString().c_str());
-}
-
 static void
 Usage(FILE *fp = stderr)
 {
     fprintf(fp, "%s: usage:\n%s", thisprog, usage);
-    versionInfo(fp);
+    fprintf(fp, "%s\n", upmpdcliVersionInfo().c_str());
     exit(1);
 }
 
 
-static const string dfltFriendlyName("UpMpd");
+static const string dfltFriendlyName("UpMpd-%h");
 
 ohProductDesc_t ohProductDesc = {
     // Manufacturer
@@ -143,7 +123,9 @@ ohProductDesc_t ohProductDesc = {
     // Product
     {
         "Upmpdcli",                                 // name
-        UPMPDCLI_PACKAGE_VERSION,                   // info
+        // Can't use g_upmpdcli_package_version here because it's in an other file and may not be
+        // initialized yet (legit chrashes with gcc15).
+        UPMPDCLI_VERSION,                           // info
         "",                                         // url
         ""                                          // imageUri
     }
@@ -153,23 +135,24 @@ ohProductDesc_t ohProductDesc = {
 static vector<UpnpDevice *> devs;
 static MPDCli *mpdclip{nullptr};
 
-string g_datadir(DATADIR "/");
+string g_datadir;
 string g_cachedir("/var/cache/upmpdcli");
 
 // Global
 string g_configfilename;
-ConfSimple *g_config;
 ConfSimple *g_state;
 bool g_enableL16 = true;
 bool g_lumincompat = false;
+bool g_mainShouldExit{false};
+std::string g_npupnpwebdocroot{"1"};
 
 static void onsig(int)
 {
     LOGDEB("Got sig" << endl);
+    g_mainShouldExit = true;
     for (auto& dev : devs) {
-        // delete has a tendancy to crash (it works most of the time
-        // though). Anyway, we're exiting, so just call shouldExit()
-        // which will send the byebyes.
+        // delete has a tendancy to crash (it works most of the time though). Anyway, we're exiting,
+        // so just call shouldExit() which will send the byebyes.
         //delete dev;
         dev->shouldExit();
     }
@@ -207,6 +190,11 @@ bool startMediaServer(bool enable)
         msroot ? nullptr : rootdevice, string("uuid:")+ uuidMS, fnameMS, enable);
     if (nullptr == mediaserver)
         return false;
+    auto lib = LibUPnP::getLibUPnP();
+    if (!g_npupnpwebdocroot.empty() && path_isabsolute(g_npupnpwebdocroot)) {
+        lib->setWebServerDocumentRoot(g_npupnpwebdocroot);
+    }
+    
     devs.push_back(mediaserver);
     LOGDEB("Media server event loop" << endl);
     // msonly && !enableMediaServer is possible if we're just using
@@ -217,55 +205,62 @@ bool startMediaServer(bool enable)
     return true;
 }
 
+// Configuration/option values. Command line have precedence, then configuration file, then
+// environment. Command line option set values in the g_lineconfig object. g_config comes from the
+// configuration file or is empty (both are non-null).
+static ConfSimple *g_config;
+static ConfSimple *g_lineconfig;
+bool getOptionValue(const std::string& nm, std::string& value, const std::string& dflt)
+{
+    if (g_lineconfig->get(nm, value))
+        return true;
+    if (g_config->get(nm, value))
+        return true;
+    std::string envar = std::string("UPMPD_") + stringtoupper(nm);
+    const char *cp = getenv(envar.c_str());
+    if (cp) {
+        value = cp;
+        return true;
+    }
+    value = dflt;
+    return false;
+}
+bool getBoolOptionValue(const std::string& nm, bool dflt)
+{
+    std::string value;
+    if (!getOptionValue(nm, value) || value.empty()) {
+        return dflt;
+    }
+    return stringToBool(value);
+}
+int getIntOptionValue(const std::string& nm, int dflt)
+{
+    std::string value;
+    if (!getOptionValue(nm, value) || value.empty()) {
+        return dflt;
+    }
+    return atoi(value.c_str());
+}
+
 int main(int argc, char *argv[])
 {
-    // Path for the sc2mpd command, or empty
-    string sc2mpdpath;
-    string screceiverstatefile;
-
-    // Sender mode: path for the command creating the mpd and mpd2sc
-    // processes, and port for the auxiliary mpd.
-    string senderpath;
-    int sendermpdport = 6700;
-
-    // Main MPD parameters
-    string mpdhost("localhost");
-    int mpdport = 6600;
-    string mpdpassword;
-
-    string logfilename;
-    int loglevel(Logger::LLINF);
-    string friendlyname(dfltFriendlyName);
-    bool ownqueue = true;
-    bool enableAV = true;
-    bool enableOH = true;
-    bool enableMediaServer = false;
-    bool ohmetapersist = true;
-    string upmpdcliuser("upmpdcli");
-    string pidfilename("/var/run/upmpdcli.pid");
-    string iconpath(DATADIR "/icon.png");
-    string presentationhtml(DATADIR "/presentation.html");
-    string iface;
-    unsigned short upport = 0;
-    string upnpip;
     int msm = 0;
-    bool inprocessms{false};
-    bool msonly{false};
-    
     const char *cp;
-    if ((cp = getenv("UPMPD_HOST")))
-        mpdhost = cp;
-    if ((cp = getenv("UPMPD_PORT")))
-        mpdport = atoi(cp);
-    if ((cp = getenv("UPMPD_FRIENDLYNAME")))
-        friendlyname = atoi(cp);
+
     if ((cp = getenv("UPMPD_CONFIG")))
         g_configfilename = cp;
-    if ((cp = getenv("UPMPD_UPNPIFACE")))
-        iface = cp;
-    if ((cp = getenv("UPMPD_UPNPPORT")))
-        upport = atoi(cp);
 
+    // Old environment variable names, this are also looked up as UPMPD_MPDHOST and UPMPD_MPDPORT
+    // Kept here for compatibility, they have the lowest priority level (overriden by command line,
+    // configuration file and new environment name)
+    string mpdhost;
+    if ((cp = getenv("UPMPD_HOST")))
+        mpdhost = cp;
+    int mpdport{6600};
+    if ((cp = getenv("UPMPD_PORT")))
+        mpdport = atoi(cp);
+
+    g_lineconfig = new ConfSimple(0, true);
     thisprog = argv[0];
     argc--; argv++;
     while (argc > 0 && **argv == '-') {
@@ -274,37 +269,34 @@ int main(int argc, char *argv[])
             Usage();
         while (**argv)
             switch (*(*argv)++) {
+                // Options without a configuration or environment equivalent
             case 'c':   op_flags |= OPT_c; if (argc < 2)  Usage();
                 g_configfilename = *(++argv); argc--; goto b1;
             case 'D':   op_flags |= OPT_D; break;
-            case 'd':   op_flags |= OPT_d; if (argc < 2)  Usage();
-                logfilename = *(++argv); argc--; goto b1;
-            case 'f':   op_flags |= OPT_f; if (argc < 2)  Usage();
-                friendlyname = *(++argv); argc--; goto b1;
-            case 'h':   op_flags |= OPT_h; if (argc < 2)  Usage();
-                mpdhost = *(++argv); argc--; goto b1;
-            case 'i':   op_flags |= OPT_i; if (argc < 2)  Usage();
-                iface = *(++argv); argc--; goto b1;
-            case 'l':   op_flags |= OPT_l; if (argc < 2)  Usage();
-                loglevel = atoi(*(++argv)); argc--; goto b1;
             case 'm':   op_flags |= OPT_m; if (argc < 2)  Usage();
                 msm = atoi(*(++argv)); argc--; goto b1;
-            case 'O': {
-                op_flags |= OPT_O; 
-                if (argc < 2)  Usage();
-                const char *cp =  *(++argv);
-                if (*cp == '1' || *cp == 't' || *cp == 'T' || *cp == 'y' || 
-                    *cp == 'Y')
-                    enableOH = true;
-                argc--; goto b1;
-            }
-            case 'P':   op_flags |= OPT_P; if (argc < 2)  Usage();
-                upport = atoi(*(++argv)); argc--; goto b1;
-            case 'p':   op_flags |= OPT_p; if (argc < 2)  Usage();
-                mpdport = atoi(*(++argv)); argc--; goto b1;
-            case 'q':   op_flags |= OPT_q; if (argc < 2)  Usage();
-                ownqueue = atoi(*(++argv)) != 0; argc--; goto b1;
-            case 'v': versionInfo(stdout); exit(0); break;
+            case 'v': std::cout << upmpdcliVersionInfo() << "\n"; return 0;
+                
+                // Options superceding config and env
+            case 'd': if (argc < 2)  Usage();
+                g_lineconfig->set("logfilename", *(++argv)); argc--; goto b1;
+            case 'f': if (argc < 2)  Usage();
+                g_lineconfig->set("friendlyname", *(++argv)); argc--; goto b1;
+            case 'h': if (argc < 2)  Usage();
+                g_lineconfig->set("mpdhost", *(++argv)); argc--; goto b1;
+            case 'i': if (argc < 2)  Usage();
+                g_lineconfig->set("upnpiface",  *(++argv)); argc--; goto b1;
+            case 'l': if (argc < 2)  Usage();
+                g_lineconfig->set("loglevel", *(++argv)); argc--; goto b1;
+            case 'O': if (argc < 2)  Usage();
+                g_lineconfig->set("openhome", *(++argv)); argc--; goto b1;
+            case 'P': if (argc < 2)  Usage();
+                g_lineconfig->set("upnpport", *(++argv)); argc--; goto b1;
+            case 'p': if (argc < 2)  Usage();
+                g_lineconfig->set("mpdport", *(++argv)); argc--; goto b1;
+            case 'q': if (argc < 2)  Usage();
+                g_lineconfig->set("ownqueue", *(++argv)); argc--; goto b1;
+
             default: Usage();   break;
             }
     b1: argc--; argv++;
@@ -318,98 +310,13 @@ int main(int argc, char *argv[])
     UpMpd::Options opts;
 
     if (!g_configfilename.empty()) {
-        g_config = new ConfSimple(g_configfilename.c_str(), 1, true);
+        g_config = new ConfSimple(
+            ConfSimple::CFSF_NOCASE|ConfSimple::CFSF_RO|ConfSimple::CFSF_TILDEXP, g_configfilename);
         if (!g_config || !g_config->ok()) {
             cerr << "Could not open config: " << g_configfilename << endl;
             return 1;
         }
-
-        string value;
-        if (!(op_flags & OPT_d))
-            g_config->get("logfilename", logfilename);
-        if (!(op_flags & OPT_f))
-            g_config->get("friendlyname", friendlyname);
-        if (!(op_flags & OPT_l)) {
-            loglevel = g_config->getInt("loglevel", Logger::LLINF);
-        }
-        if (!(op_flags & OPT_h)) {
-            g_config->get("mpdhost", mpdhost);
-        }
-        if (!(op_flags & OPT_p)) {
-            mpdport =  g_config->getInt("mpdport", 6600);
-        }
-        g_config->get("mpdpassword", mpdpassword);
-        if (!(op_flags & OPT_q)) {
-            ownqueue = g_config->getBool("ownqueue", true);
-        }
-        enableOH = g_config->getBool("openhome", true);
-        enableAV = g_config->getBool("upnpav", true);
-
-        if (!g_config->getBool("checkcontentformat", true)) {
-            // If option is specified and 0, set nocheck flag
-            opts.options |= UpMpd::upmpdNoContentFormatCheck;
-        }
-        ohmetapersist = g_config->getBool("ohmetapersist", true);
-        if (g_config->get("pkgdatadir", g_datadir)) {
-            path_catslash(g_datadir);
-            iconpath = path_cat(g_datadir, "icon.png");
-            if (!path_exists(iconpath)) {
-                iconpath.clear();
-            }
-            presentationhtml = path_cat(g_datadir, "presentation.html");
-        }
-        g_config->get("iconpath", iconpath);
-        g_config->get("presentationhtml", presentationhtml);
-        g_config->get("cachedir", opts.cachedir);
-        g_config->get("pidfile", pidfilename);
-        if (!(op_flags & OPT_i)) {
-            g_config->get("upnpiface", iface);
-            if (iface.empty()) {
-                g_config->get("upnpip", upnpip);
-            }
-        }
-        if (!(op_flags & OPT_P)) {
-            upport = g_config->getInt("upnpport", 0);
-        }
-        opts.schttpport = g_config->getInt("schttpport", 0);
-        g_config->get("scplaymethod", opts.scplaymethod);
-        g_config->get("sc2mpd", sc2mpdpath);
-        g_config->get("screceiverstatefile", screceiverstatefile);
-        if (g_config->getBool("scnosongcastsource", false)) {
-            // If option is specified and 1, set nocheck flag
-            opts.options |= UpMpd::upmpdNoSongcastSource;
-        }
-        opts.ohmetasleep = g_config->getInt("ohmetasleep", 0);
-        g_config->get("ohmanufacturername", ohProductDesc.manufacturer.name);
-        g_config->get("ohmanufacturerinfo", ohProductDesc.manufacturer.info);
-        g_config->get("ohmanufacturerurl", ohProductDesc.manufacturer.url);
-        g_config->get("ohmanufacturerimageuri", ohProductDesc.manufacturer.imageUri);
-        g_config->get("ohmodelname", ohProductDesc.model.name);
-        g_config->get("ohmodelinfo", ohProductDesc.model.info);
-        g_config->get("ohmodelurl", ohProductDesc.model.url);
-        // imageUri was mistake, keep compat and override with imageuri if set
-        g_config->get("ohmodelimageUri", ohProductDesc.model.imageUri);
-        g_config->get("ohmodelimageuri", ohProductDesc.model.imageUri);
-        g_config->get("ohproductname", ohProductDesc.product.name);
-        g_config->get("ohproductinfo", ohProductDesc.product.info);
-        g_config->get("ohproducturl", ohProductDesc.product.url);
-        g_config->get("ohproductimageuri", ohProductDesc.product.imageUri);
-        g_config->get("ohproductroom", ohProductDesc.room);
-        // ProductName is set to ModelName by default
-        if (ohProductDesc.product.name.empty()) {
-            ohProductDesc.product.name = ohProductDesc.model.name;
-        }
-        // ProductRoom is set to "Main Room" by default
-        if (ohProductDesc.room.empty()) {
-            ohProductDesc.room = "Main Room";
-        }
-
-        g_config->get("scsenderpath", senderpath);
-        if (g_config->get("scsendermpdport", value))
-            sendermpdport = atoi(value.c_str());
-
-        g_lumincompat = g_config->getBool("lumincompat", g_lumincompat);
-    } else {
+    }  else {
         // g_configfilename is empty. Create an empty config anyway
         g_config = new ConfSimple(string(), 1, true);
         if (!g_config || !g_config->ok()) {
@@ -417,9 +324,108 @@ int main(int argc, char *argv[])
             return 1;
         }
     }
+        
+    string value;
 
+    string logfilename;
+    getOptionValue("logfilename", logfilename);
+    string friendlyname;
+    getOptionValue("friendlyname", friendlyname, dfltFriendlyName);
+    getOptionValue("mpdhost", mpdhost, "localhost");
+    string iface;
+    getOptionValue("upnpiface", iface);
+    string upnpip;
+    if (iface.empty()) {
+        getOptionValue("upnpip", upnpip);
+    }
+    int loglevel = getIntOptionValue("loglevel", Logger::LLINF);
+    bool enableOH = getBoolOptionValue("openhome", true);
+    bool enableAV = getBoolOptionValue("upnpav", true);
+    unsigned short upport = getIntOptionValue("upnpport", 0);
+    if (getOptionValue("mpdport", value) && !value.empty()) {
+        mpdport = atoi(value.c_str());
+    }
+    bool ownqueue = getBoolOptionValue("ownqueue", true);
+    string mpdpassword;
+    getOptionValue("mpdpassword", mpdpassword);
+    opts.options |= UpMpd::upmpdNoContentFormatCheck;
+    if (getOptionValue("checkcontentformat", value) && !value.empty() && stringToBool(value)) {
+        // If option is specified and 1, unset nocheck flag
+        opts.options &= ~UpMpd::upmpdNoContentFormatCheck;
+    }
+    bool ohmetapersist = getBoolOptionValue("ohmetapersist", true);
+
+    getOptionValue("pkgdatadir", g_datadir, DATADIR);
+    if (g_datadir.empty()) {
+        // Built as portable install. Use the executable name to compute a likely location
+        auto bindir = path_thisexecdir();
+        g_datadir = path_cat(path_getfather(bindir), {"share", "upmpdcli"});
+    }
+    path_catslash(g_datadir);
+    string iconpath = path_cat(g_datadir, "icon.png");
+    if (!path_exists(iconpath)) {
+        iconpath.clear();
+    }
+    string presentationhtml = path_cat(g_datadir, "presentation.html");
+
+    getOptionValue("iconpath", iconpath);
+    getOptionValue("presentationhtml", presentationhtml);
+    getOptionValue("cachedir", opts.cachedir);
+    string pidfilename;
+    getOptionValue("pidfile", pidfilename, "/var/run/upmpdcli.pid");
+
+    opts.schttpport = getIntOptionValue("schttpport", 0);
+    getOptionValue("scplaymethod", opts.scplaymethod);
+    // Path for the sc2mpd command, or empty
+    string sc2mpdpath;
+    getOptionValue("sc2mpd", sc2mpdpath);
+    string screceiverstatefile;
+    getOptionValue("screceiverstatefile", screceiverstatefile);
+    if (getOptionValue("scnosongcastsource", value) && !value.empty() && stringToBool(value)) {
+        // If option is specified and 1, set flag
+        opts.options |= UpMpd::upmpdNoSongcastSource;
+    }
+    opts.ohmetasleep = getIntOptionValue("ohmetasleep", 0);
+    getOptionValue("ohmanufacturername", ohProductDesc.manufacturer.name);
+    getOptionValue("ohmanufacturerinfo", ohProductDesc.manufacturer.info);
+    getOptionValue("ohmanufacturerurl", ohProductDesc.manufacturer.url);
+    getOptionValue("ohmanufacturerimageuri", ohProductDesc.manufacturer.imageUri);
+    getOptionValue("ohmodelname", ohProductDesc.model.name);
+    getOptionValue("ohmodelinfo", ohProductDesc.model.info);
+    getOptionValue("ohmodelurl", ohProductDesc.model.url);
+    // imageUri was mistake, keep compat and override with imageuri if set
+    getOptionValue("ohmodelimageUri", ohProductDesc.model.imageUri);
+    getOptionValue("ohmodelimageuri", ohProductDesc.model.imageUri);
+    getOptionValue("ohproductname", ohProductDesc.product.name);
+    getOptionValue("ohproductinfo", ohProductDesc.product.info);
+    getOptionValue("ohproducturl", ohProductDesc.product.url);
+    getOptionValue("ohproductimageuri", ohProductDesc.product.imageUri);
+    getOptionValue("ohproductroom", ohProductDesc.room);
+    // ProductName is set to ModelName by default
+    if (ohProductDesc.product.name.empty()) {
+        ohProductDesc.product.name = ohProductDesc.model.name;
+    }
+    // ProductRoom is set to "Main Room" by default
+    if (ohProductDesc.room.empty()) {
+        ohProductDesc.room = "Main Room";
+    }
+    // Sender mode: path for the command creating the mpd and mpd2sc
+    // processes, and port for the auxiliary mpd.
+    string senderpath;
+    getOptionValue("scsenderpath", senderpath);
+    int sendermpdport = getIntOptionValue("scsendermpdport", 6700);
+    g_lumincompat = getBoolOptionValue("lumincompat", false);
+
+    // If we are root, and the log file exists, we need to temporarily chown it to root, because it
+    // may not be writable else (e.g. sticky world-writable /tmp)
+    if (geteuid() == 0 && !logfilename.empty() && logfilename.compare("stderr") &&
+        path_exists(logfilename)) {
+        if (chown(logfilename.c_str(), 0, -1) < 0 && errno != ENOENT) {
+            std::cerr << "chown(" << logfilename << ") : errno : " << errno << '\n';
+        }
+    }
     if (Logger::getTheLog(logfilename) == 0) {
-        cerr << "Can't initialize log" << endl;
+        std::cerr << "Can't initialize log" << '\n';
         return 1;
     }
     Logger::getTheLog("")->reopen(logfilename);
@@ -429,7 +435,9 @@ int main(int argc, char *argv[])
     // Server. We let a static ContentDirectory method decide this
     // for us. The way we then implement it depends on the command
     // line option (see the enum comments near the top of the file):
-    enableMediaServer = ContentDirectory::mediaServerNeeded();
+    bool inprocessms{false};
+    bool msonly{false};
+    bool enableMediaServer = ContentDirectory::mediaServerNeeded();
     switch (arg_msmode) {
     case MSOnly:
         inprocessms = true;
@@ -464,6 +472,7 @@ int main(int argc, char *argv[])
 
     // If started by root, we use the pidfile and we will change the
     // uid (later). First part follows
+    string upmpdcliuser("upmpdcli");
     uid_t runas(0);
     gid_t runasg(0);
     struct passwd *pass = getpwnam(upmpdcliuser.c_str());
@@ -520,6 +529,17 @@ int main(int argc, char *argv[])
 
     string statefn = path_cat(opts.cachedir, "/upmstate");
     g_state = new ConfSimple(statefn.c_str());
+
+    std::string docroot;
+    if (getOptionValue("webserverdocumentroot", docroot)) {
+        if (docroot == "1") {
+            g_npupnpwebdocroot = path_cat(g_cachedir, "www");
+        } else if (path_isabsolute(docroot)) {
+            g_npupnpwebdocroot = docroot;
+        } else {
+            g_npupnpwebdocroot.clear();
+        }
+    }
     
     opts.cachefn.clear();
     if (!msonly && ohmetapersist) {
@@ -626,52 +646,14 @@ int main(int argc, char *argv[])
         }
     }
 
-
-    // Initialize MPD client object. Retry until it works or power fail.
-    if (!msonly) {
-        int mpdretrysecs = 2;
-        for (;;) {
-            mpdclip = new MPDCli(mpdhost, mpdport, mpdpassword);
-            if (mpdclip == 0) {
-                LOGFAT("Can't allocate MPD client object" << endl);
-                return 1;
+    // Initialise lower upnp lib logging. Static so can be done before the rest of init.
+    {
+        std::string upnplogfilename;
+        if (getOptionValue("upnplogfilename", upnplogfilename)) {
+            int upnploglevel = getIntOptionValue("upnploglevel", LibUPnP::LogLevelError);
+            if (upnploglevel != LibUPnP::LogLevelNone) {
+                LibUPnP::setLogFileName(upnplogfilename, LibUPnP::LogLevel(upnploglevel));
             }
-            if (!mpdclip->ok()) {
-                LOGERR("MPD connection failed" << endl);
-                delete mpdclip;
-                mpdclip = 0;
-                sleep(mpdretrysecs);
-                mpdretrysecs = MIN(2*mpdretrysecs, 120);
-            } else {
-                break;
-            }
-        }
-        const MpdStatus& mpdstat = mpdclip->getStatus();
-        // Only the "special" upmpdcli 0.19.16 version has patch != 0
-        g_enableL16 = mpdstat.versmajor >= 1 || mpdstat.versminor >= 20 ||
-            mpdstat.verspatch >= 16;
-        // It appeared in the past that L16 is a major source of issues when
-        // playing with win10 'cast to device', inciting it to transcode for
-        // some reason, with very bad results. Can't reproduce this now. So
-        // change config default to true.
-        bool confl16{true};
-        if (g_config) {
-            confl16 = g_config->getBool("enablel16", true);
-        }
-        g_enableL16 = g_enableL16 && confl16;
-    }
-    
-    // Initialise lower upnp lib logging. Static so can be done before
-    // the rest of init.
-    if ((cp = getenv("UPMPDCLI_UPNPLOGFILENAME"))) {
-        char *cp1 = getenv("UPMPDCLI_UPNPLOGLEVEL");
-        int loglevel = LibUPnP::LogLevelNone;
-        if (cp1) {
-            loglevel = atoi(cp1);
-        }
-        loglevel = loglevel < 0 ? 0: loglevel;
-        if (loglevel != LibUPnP::LogLevelNone) {
-            LibUPnP::setLogFileName(cp, LibUPnP::LogLevel(loglevel));
         }
     }
 
@@ -693,6 +675,9 @@ int main(int argc, char *argv[])
                           LibUPnP::UPNPPINIT_OPTION_END)) {
             break;
         }
+        if (g_mainShouldExit) {
+            return 1;
+        }
         sleep(libretrysecs);
         libretrysecs = MIN(2*libretrysecs, 120);
     }
@@ -702,9 +687,12 @@ int main(int argc, char *argv[])
         return 1;
     }
     hwaddr = mylib->hwaddr();
-    
+
+    friendlyname = fnameSetup(friendlyname);
     // Create unique IDs for renderer and possible media server
-    if (!g_config || !g_config->get("msfriendlyname", fnameMS)) {
+    if (getOptionValue("msfriendlyname", fnameMS)) {
+        fnameMS = fnameSetup(fnameMS);
+    } else {
         fnameMS = friendlyname + "-mediaserver";
     }
     uuidMS = LibUPnP::makeDevUUID(fnameMS, hwaddr);
@@ -715,47 +703,80 @@ int main(int argc, char *argv[])
         pidfilename = pidfilename + "-ms";
     }
 
-    opts.iconpath = iconpath;
-    opts.presentationhtml = presentationhtml;
-    if (ownqueue)
-        opts.options |= UpMpd::upmpdOwnQueue;
-    if (enableOH)
-        opts.options |= UpMpd::upmpdDoOH;
-    if (ohmetapersist)
-        opts.options |= UpMpd::upmpdOhMetaPersist;
-    if (!sc2mpdpath.empty()) {
-        opts.sc2mpdpath = sc2mpdpath;
-        opts.options |= UpMpd::upmpdOhReceiver;
-    }
-    if (!screceiverstatefile.empty()) {
-        opts.screceiverstatefile = screceiverstatefile;
-        int fd;
-        if ((fd = open(opts.screceiverstatefile.c_str(),
-                       O_CREAT|O_RDWR, 0644)) < 0) {
-            LOGERR("creat(" << opts.screceiverstatefile << ") : errno : " << errno << endl);
-        } else {
-            close(fd);
-            if (geteuid() == 0 && chown(opts.screceiverstatefile.c_str(),
-                                        runas, -1) != 0) {
-                LOGERR("chown(" << opts.screceiverstatefile << ") : errno : " << errno << endl);
-            }
-        }
-    }
-    if (!senderpath.empty()) {
-        opts.options |= UpMpd::upmpdOhSenderReceiver;
-        opts.senderpath = senderpath;
-        opts.sendermpdport = sendermpdport;
-    }
-
-    if (!enableAV)
-        opts.options |= UpMpd::upmpdNoAV;
-
     setupsigs();
 
+    if (inprocessms && !startMediaServer(enableMediaServer)) {
+        LOGERR("Could not start media server\n");
+        std::cerr << "Could not start media server\n";
+        return 0;
+    }
+
     UpMpd *mediarenderer{nullptr};
     if (!msonly) {
-        mediarenderer = new UpMpd(hwaddr, friendlyname,
-                                  ohProductDesc, mpdclip, opts);
+        // Initialize MPD client object. Retry until it works or power fail.
+        int mpdretrysecs = 2;
+        for (;;) {
+            mpdclip = new MPDCli(mpdhost, mpdport, mpdpassword);
+            if (mpdclip == 0) {
+                LOGFAT("Can't allocate MPD client object" << endl);
+                return 1;
+            }
+            if (!mpdclip->ok()) {
+                if (g_mainShouldExit) {
+                    return 1;
+                }
+                LOGERR("MPD connection failed" << "\n");
+                delete mpdclip;
+                mpdclip = 0;
+                sleep(mpdretrysecs);
+                mpdretrysecs = MIN(2*mpdretrysecs, 120);
+            } else {
+                break;
+            }
+        }
+        const MpdStatus& mpdstat = mpdclip->getStatus();
+        // Only the "special" upmpdcli 0.19.16 version has patch != 0
+        g_enableL16 = mpdstat.versmajor >= 1 || mpdstat.versminor >= 20 || mpdstat.verspatch >= 16;
+        // It appeared in the past that L16 is a major source of issues when
+        // playing with win10 'cast to device', inciting it to transcode for
+        // some reason, with very bad results. Can't reproduce this now. So
+        // change config default to true.
+        bool confl16 = getBoolOptionValue("enablel16", true);
+        g_enableL16 = g_enableL16 && confl16;
+
+        opts.iconpath = iconpath;
+        opts.presentationhtml = presentationhtml;
+        if (ownqueue)
+            opts.options |= UpMpd::upmpdOwnQueue;
+        if (enableOH)
+            opts.options |= UpMpd::upmpdDoOH;
+        if (ohmetapersist)
+            opts.options |= UpMpd::upmpdOhMetaPersist;
+        if (!sc2mpdpath.empty()) {
+            opts.sc2mpdpath = sc2mpdpath;
+            opts.options |= UpMpd::upmpdOhReceiver;
+        }
+        if (!screceiverstatefile.empty()) {
+            opts.screceiverstatefile = screceiverstatefile;
+            int fd;
+            if ((fd = open(opts.screceiverstatefile.c_str(), O_CREAT|O_RDWR, 0644)) < 0) {
+                LOGSYSERR("main", "open/create", opts.screceiverstatefile);
+            } else {
+                close(fd);
+                if (geteuid() == 0 && chown(opts.screceiverstatefile.c_str(), runas, -1) != 0) {
+                    LOGSYSERR("main", "chown", opts.screceiverstatefile);
+                }
+            }
+        }
+        if (!senderpath.empty()) {
+            opts.options |= UpMpd::upmpdOhSenderReceiver;
+            opts.senderpath = senderpath;
+            opts.sendermpdport = sendermpdport;
+        }
+
+        if (!enableAV)
+            opts.options |= UpMpd::upmpdNoAV;
+        mediarenderer = new UpMpd(hwaddr, friendlyname, ohProductDesc, mpdclip, opts);
         UpMpdOpenHome *oh = mediarenderer->getoh();
         // rootdevice is only used if we implement the media server as
         // an embedded device, which is mostly for testing purposes
@@ -769,15 +790,6 @@ int main(int argc, char *argv[])
             rootdevice = av;
             devs.push_back(av);
         }
-    }
-
-    if (inprocessms && !startMediaServer(enableMediaServer)) {
-        LOGERR("Could not start media server\n");
-        std::cerr << "Could not start media server\n";
-        return 0;
-    }
-
-    if (!msonly) {
         LOGDEB("Renderer event loop" << endl);
         mediarenderer->startnoloops();
     }
